#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include <winbase.h>
#include <iostream.h>
#include <string.h>
#include <Shlwapi.h>

/*
	IMAGE-FILE VIEWER PROXY
	-----------------------
	The proxy is associated with image-file types. When invoked, if the filename contains "ID-<identifier>" at the end (before
	the file extension) then a genealogical program is launched with the arguments <image-path> and <identifier>, otherwise a
	normal image viewer is launched with <image-path>.

	Author:		A. C. Proctor	10-Dec-2015
			written in Notepad without a debugger, so apologies for any typos.

	To compile:	cl proxy.c /link shlwapi.lib user32.lib -out:proxy.exe
*/

#ifndef INVALID_FILE_ATTRIBUTES
#define INVALID_FILE_ATTRIBUTES ((DWORD)-1)
#endif

/*
	Although intended as a sample proxy for image files, this code uses text files for
	demonstration purposes (notepad for normal files, and wordpad for genealogical files), and
	displays diagnostic messages. These can all be changed for a real proxy.
*/
const TCHAR geneApp[] = TEXT("C:\\Program Files\\Windows NT\\Accessories\\wordpad.exe");	/* Command line: file-path file-identifier */
const TCHAR normalApp[] = TEXT("notepad");							/* Command line: file-path */

BOOL fileExists(LPCTSTR szPath)
/* Test whether specified path is an existing file, and not a directory */
{
    DWORD dwAttrib = GetFileAttributes(szPath);

    return (dwAttrib != INVALID_FILE_ATTRIBUTES && !(dwAttrib & FILE_ATTRIBUTE_DIRECTORY));
}


void getId (LPCTSTR szPath, LPTSTR id)
/* Locates and returns any file identifier beginning "ID-" in the filename part of the specified path */
{
    LPTSTR fileExtPart = PathFindFileName(szPath);
    TCHAR filePart [MAX_PATH+1];

    StrCpy (filePart, fileExtPart);
    PathRemoveExtension (filePart);
    _tprintf (TEXT("File name: %s\n"), filePart);

    id[0] = 0;
    StrCpy (id, strstr(filePart,TEXT("ID-")));
    _tprintf (TEXT("File ID: %s\n"), id);
}

void runApp (LPCTSTR szFilePath, LPCTSTR szId)
/* Runs the appropriate application for the given file and an optional genealogical identifier */
{
    TCHAR cmdLine[3*MAX_PATH+8];
    PROCESS_INFORMATION processInformation;
    STARTUPINFO startupInfo;

    if (szId != NULL && strlen(szId) > 0)
    {
	sprintf (cmdLine,"\"%s\" \"%s\" \"%s\"", geneApp, szFilePath, szId);
    }
    else
    {
	sprintf (cmdLine,"\"%s\" \"%s\"", normalApp, szFilePath);
    }
    _tprintf (TEXT("Executing: %s\n"),cmdLine);

    memset (&processInformation, 0, sizeof(processInformation));
    memset (&startupInfo, 0, sizeof(startupInfo));
    startupInfo.cb = sizeof(startupInfo);

    if (!CreateProcess(NULL, cmdLine, NULL, NULL, FALSE, NORMAL_PRIORITY_CLASS, NULL, NULL, &startupInfo, &processInformation))
    {
	MessageBox (NULL, "Failed to launch application", NULL, MB_OK);
    }
}

int _tmain (int argc, TCHAR *argv[])
{
    TCHAR fileId[MAX_PATH+1];

    if( argc != 2 )
    {
	_tprintf (TEXT("Usage: %s file-path\n"), argv[0]);
	return 1;
    }

    _tprintf (TEXT("Looking for file: %s\n"), argv[1]);

    if (fileExists(argv[1])) 
    {
	_tprintf (TEXT("File %s exists\n"), argv[1]);
	getId (argv[1], fileId);
    } 
    else 
    {
	_tprintf (TEXT("File %s doesn't exist\n"), argv[1]);
	fileId[0] = 0;
    }

    runApp (argv[1], fileId);
    return 0;
}